import Flutter
import UIKit
import PanoRtc

public class SwiftPanoRtcPlugin: NSObject, FlutterPlugin, FlutterStreamHandler {
    private weak var registrar: FlutterPluginRegistrar?
    private var methodChannel: FlutterMethodChannel?
    private var eventChannel: FlutterEventChannel?
    private var eventSink: FlutterEventSink? = nil
    private lazy var manager: RtcEngineManager = {
        let `manager` = RtcEngineManager()
        manager.delegate = self
        return manager
    }()
    private lazy var whiteboardPlugins = Array<PanoRtcWhiteboardPlugin>()
    private var videoStreamManagerPlugin: PanoRtcVideoStreamManagerPlugin?
    private(set) var annotationManagerPlugin: PanoRtcAnnotationManagerPlugin?
    private var networkManagerPlugin: PanoRtcNetworkManagerPlugin?
    
    weak var engineManager: RtcEngineManager? {
        return manager
    }
    
    public static func register(with registrar: FlutterPluginRegistrar) {
        let rtcEnginePlugin = SwiftPanoRtcPlugin()
        rtcEnginePlugin.initPlugin(registrar)
    }
    
    private func initPlugin(_ registrar: FlutterPluginRegistrar) {
        self.registrar = registrar
        methodChannel = FlutterMethodChannel(name: "pano_rtc/api_engine", binaryMessenger: registrar.messenger())
        eventChannel = FlutterEventChannel(name: "pano_rtc/events_engine", binaryMessenger: registrar.messenger())
        registrar.addMethodCallDelegate(self, channel: methodChannel!)
        eventChannel?.setStreamHandler(self)
        
        let viewFactory = PanoNativeViewFactory(registrar.messenger(), self)
        registrar.register(viewFactory, withId: "PanoNativeView")
        
        let whiteboardViewFactory = PanoWhiteboardNativeViewFactory(registrar.messenger(), self)
        registrar.register(whiteboardViewFactory, withId: "PanoWhiteboardNativeView")
    }
    
    public func detachFromEngine(for registrar: FlutterPluginRegistrar) {
        manager.Release()
        
        methodChannel?.setMethodCallHandler(nil)
        eventChannel?.setStreamHandler(nil)
        
        for whiteboardPlugin in whiteboardPlugins {
            whiteboardPlugin.detachFromEngine(for: registrar)
        }
        whiteboardPlugins.removeAll()
        
        videoStreamManagerPlugin?.detachFromEngine(for: registrar)
        videoStreamManagerPlugin = nil
        
        annotationManagerPlugin?.detachFromEngine(for: registrar)
        annotationManagerPlugin = nil
        
        networkManagerPlugin?.detachFromEngine(for: registrar)
        networkManagerPlugin = nil
    }

    public func onListen(withArguments arguments: Any?, eventSink events: @escaping FlutterEventSink) -> FlutterError? {
        eventSink = events
        return nil
    }

    public func onCancel(withArguments arguments: Any?) -> FlutterError? {
        eventSink = nil
        return nil
    }
    
    public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        if let params = call.arguments as? NSDictionary {
            let selector = NSSelectorFromString(call.method + "::")
            if manager.responds(to: selector) {
                manager.perform(selector, with: params, with: ResultCallback(result))
                return
            }
        } else {
            let selector = NSSelectorFromString(call.method + ":")
            if manager.responds(to: selector) {
                manager.perform(selector, with: ResultCallback(result))
                return
            }
        }
        result(FlutterMethodNotImplemented)
    }
}

extension SwiftPanoRtcPlugin: RtcEngineManagerDelegate {
    internal func emit(_ methodName: String, _ data: Dictionary<String, Any?>?) {
        var event: Dictionary<String, Any?> = ["methodName": methodName]
        if let `data` = data {
            event.merge(data) { (current, _) in
                current
            }
        }
        eventSink?(event)
    }
    
    func createWhiteboardEngine(id: String, panoObj: PanoRtcWhiteboard?) -> RtcWhiteboard? {
        guard let `panoObj` = panoObj else {
            return nil
        }
        let whiteboard = RtcWhiteboard(id: id, whiteboardEngine: panoObj)
        let whiteboardPlugin = PanoRtcWhiteboardPlugin(registrar!, whiteboardId: id, innerObj: whiteboard)
        whiteboardPlugin.initPlugin()
        whiteboardPlugins.append(whiteboardPlugin)
        return whiteboard
    }
    
    func createVideoStreamManager(panoObj: PanoRtcVideoStreamManager?) -> RtcVideoStreamManager? {
        guard let `panoObj` = panoObj else {
            return nil
        }
        let videoStreamManager = RtcVideoStreamManager(manager: panoObj)
        videoStreamManagerPlugin = PanoRtcVideoStreamManagerPlugin(registrar!, innerObj: videoStreamManager)
        videoStreamManagerPlugin?.initPlugin()
        return videoStreamManager
    }
    
    func createAnnotationManager(panoObj: PanoRtcAnnotationManager?) -> RtcAnnotationManager? {
        guard let `panoObj` = panoObj else {
            return nil
        }
        let annotationManager = RtcAnnotationManager(manager: panoObj)
        annotationManagerPlugin = PanoRtcAnnotationManagerPlugin(registrar!, innerObj: annotationManager)
        annotationManagerPlugin?.initPlugin()
        return annotationManager
    }
    
    func createNetworkManager(panoObj: PanoRtcEngineKit?) -> RtcNetworkManager? {
        guard let `panoObj` = panoObj else {
            return nil
        }
        let networkManager = RtcNetworkManager(engine: panoObj)
        networkManagerPlugin = PanoRtcNetworkManagerPlugin(registrar!, innerObj: networkManager)
        networkManagerPlugin?.initPlugin()
        return networkManager
    }
}
