import 'dart:async';

import 'package:flutter/foundation.dart';
import 'package:flutter/widgets.dart';
import 'package:flutter/rendering.dart';
import 'package:flutter/services.dart';

import '../pano_rtc.dart';
import 'enum_converter.dart';

typedef void RtcSurfaceViewCreatedCallback(RtcSurfaceViewModel viewModel);

/// The RtcSurfaceViewModel class.
class RtcSurfaceViewModel {
  final MethodChannel _methodChannel;

  Future<T> invokeMethod<T>(String method, [Map<String, dynamic> arguments]) {
    if (T == ResultCode) {
      return _methodChannel.invokeMethod(method, arguments).then((value) {
        return ResultCodeConverter.fromValue(value).e as T;
      });
    } else {
      return _methodChannel.invokeMethod(method, arguments);
    }
  }

  RtcSurfaceViewModel(this._methodChannel);
}

class RtcSurfaceView extends StatefulWidget {
  final RtcSurfaceViewCreatedCallback onViewCreated;

  const RtcSurfaceView({Key key, this.onViewCreated}) : super(key: key);

  @override
  State<StatefulWidget> createState() => _RtcSurfaceViewState();
}

class _RtcSurfaceViewState extends State<RtcSurfaceView> {
  final Completer<RtcSurfaceViewModel> _viewModel =
      Completer<RtcSurfaceViewModel>();

  @override
  Widget build(BuildContext context) {
    if (defaultTargetPlatform == TargetPlatform.iOS) {
      return GestureDetector(
        behavior: HitTestBehavior.opaque,
        child: UiKitView(
          viewType: 'PanoNativeView',
          onPlatformViewCreated: onPlatformViewCreated,
          hitTestBehavior: PlatformViewHitTestBehavior.transparent,
          creationParams: {},
          creationParamsCodec: const StandardMessageCodec(),
        ),
      );
    } else if (defaultTargetPlatform == TargetPlatform.android) {
      return GestureDetector(
        behavior: HitTestBehavior.opaque,
        child: AndroidView(
            viewType: 'PanoNativeView',
            onPlatformViewCreated: onPlatformViewCreated,
            hitTestBehavior: PlatformViewHitTestBehavior.transparent,
            creationParams: {},
            creationParamsCodec: const StandardMessageCodec()),
      );
    }
    return Text('$defaultTargetPlatform is not yet supported by the plugin');
  }

  Future<void> onPlatformViewCreated(int id) async {
    final RtcSurfaceViewModel viewModel =
        RtcSurfaceViewModel(MethodChannel('pano_rtc/view_surface_$id'));
    _viewModel.complete(viewModel);
    if (widget.onViewCreated != null) {
      widget.onViewCreated(viewModel);
    }
  }
}

typedef void RtcWhiteboardSurfaceViewCreatedCallback(
    RtcWhiteboardSurfaceViewModel viewModel);

/// The RtcWhiteboardSurfaceViewModel class.
class RtcWhiteboardSurfaceViewModel {
  final MethodChannel _methodChannel;

  Future<T> invokeMethod<T>(String method, [Map<String, dynamic> arguments]) {
    if (T == ResultCode) {
      return _methodChannel.invokeMethod(method, arguments).then((value) {
        return ResultCodeConverter.fromValue(value).e as T;
      });
    } else {
      return _methodChannel.invokeMethod(method, arguments);
    }
  }

  RtcWhiteboardSurfaceViewModel(this._methodChannel);
}

class RtcWhiteboardSurfaceView extends StatefulWidget {
  final RtcWhiteboardSurfaceViewCreatedCallback onViewCreated;

  const RtcWhiteboardSurfaceView({Key key, this.onViewCreated})
      : super(key: key);

  @override
  State<StatefulWidget> createState() => _RtcWhiteboardSurfaceViewState();
}

class _RtcWhiteboardSurfaceViewState extends State<RtcWhiteboardSurfaceView> {
  final Completer<RtcWhiteboardSurfaceViewModel> _viewModel =
      Completer<RtcWhiteboardSurfaceViewModel>();

  @override
  Widget build(BuildContext context) {
    if (defaultTargetPlatform == TargetPlatform.iOS) {
      return GestureDetector(
        behavior: HitTestBehavior.opaque,
        child: UiKitView(
          viewType: 'PanoWhiteboardNativeView',
          onPlatformViewCreated: onPlatformViewCreated,
          hitTestBehavior: PlatformViewHitTestBehavior.transparent,
          creationParams: {},
          creationParamsCodec: const StandardMessageCodec(),
        ),
      );
    } else if (defaultTargetPlatform == TargetPlatform.android) {
      return GestureDetector(
        behavior: HitTestBehavior.opaque,
        child: AndroidView(
            viewType: 'PanoWhiteboardNativeView',
            onPlatformViewCreated: onPlatformViewCreated,
            hitTestBehavior: PlatformViewHitTestBehavior.transparent,
            creationParams: {},
            creationParamsCodec: const StandardMessageCodec()),
      );
    }
    return Text('$defaultTargetPlatform is not yet supported by the plugin');
  }

  Future<void> onPlatformViewCreated(int id) async {
    final RtcWhiteboardSurfaceViewModel viewModel =
        RtcWhiteboardSurfaceViewModel(
            MethodChannel('pano_rtc/whiteboard_surface_view_$id'));
    _viewModel.complete(viewModel);
    if (widget.onViewCreated != null) {
      widget.onViewCreated(viewModel);
    }
  }
}
